@@ -4,10 +4,11 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Calendar;
 import java.util.Date;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 
 import org.apache.commons.lang.NullArgumentException;
-import org.gcube.common.core.utils.logging.GCUBEClientLog;
 import org.gcube.portal.databook.shared.Comment;
 import org.gcube.portal.databook.shared.Feed;
 import org.gcube.portal.databook.shared.FeedType;
@@ -25,6 +26,8 @@ import org.gcube.portal.databook.shared.ex.NotificationChannelTypeNotFoundExcept
 import org.gcube.portal.databook.shared.ex.NotificationIDNotFoundException;
 import org.gcube.portal.databook.shared.ex.NotificationTypeNotFoundException;
 import org.gcube.portal.databook.shared.ex.PrivacyLevelTypeNotFoundException;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import com.netflix.astyanax.MutationBatch;
 import com.netflix.astyanax.connectionpool.OperationResult;
@@ -49,7 +52,7 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 	/**
 	 * logger
 	 */
-	static GCUBEClientLog _log = new GCUBEClientLog(DBCassandraAstyanaxImpl.class);
+	private static final Logger _log = LoggerFactory.getLogger(DBCassandraAstyanaxImpl.class);
 	/**
 	 * Column Family names
 	 */
@@ -65,9 +68,9 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 	public static final String USER_LIKED_FEEDS = "USERLikes";
 	public static final String USER_NOTIFICATIONS = "USERNotifications"; // regular user notifications timeline
 	public static final String USER_MESSAGES_NOTIFICATIONS = "USERMessagesNotifications";  // user messages notifications timeline
-	public static final String USER_NOTIFICATIONS_SETTINGS = "USERNotificationsSettings"; // settings for notifications
+	public static final String USER_NOTIFICATIONS_PREFERENCES = "USERNotificationsPreferences"; // preferences for notifications
 
-	private static ColumnFamily<String, String> cf_Connections =	new ColumnFamily<String, String>(
+	private static ColumnFamily<String, String> cf_Connections = new ColumnFamily<String, String>(
 			CONNECTIONS, // Column Family Name
 			StringSerializer.get(),   // Key Serializer
 			StringSerializer.get());  // Column Serializer
@@ -118,11 +121,13 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 			USER_MESSAGES_NOTIFICATIONS,              // Column Family Name
 			StringSerializer.get(),   // Key Serializer
 			StringSerializer.get());  // Column Serializer
-	private static ColumnFamily<String, String> cf_UserNotificationsSettings =	new ColumnFamily<String, String>(
-			USER_NOTIFICATIONS_SETTINGS,  // Column Family Name
+	private static ColumnFamily<String, String> cf_UserNotificationsPreferences =	new ColumnFamily<String, String>(
+			USER_NOTIFICATIONS_PREFERENCES,  // Column Family Name
 			StringSerializer.get(),   // Key Serializer
 			StringSerializer.get());  // Column Serializer
 
+
+
 	/**
 	 * connection instance
 	 */
@@ -904,72 +909,125 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 	 */
 	/**
 	 * {@inheritDoc}
-	 * @throws NotificationChannelTypeNotFoundException 
 	 */
 	@Override
-	public List<NotificationChannelType> getUserNotificationChannels(String userid) throws NotificationChannelTypeNotFoundException {
+	public List<NotificationChannelType> getUserNotificationChannels(String userid, NotificationType notificationType) throws NotificationChannelTypeNotFoundException, NotificationTypeNotFoundException {
+		_log.trace("Asking for Single Notification preference of  " + userid +  " Type: " + notificationType);
 		List<NotificationChannelType> toReturn = new ArrayList<NotificationChannelType>();
-		OperationResult<Rows<String, String>> result = null;
-		try {
-			result = conn.getKeyspace().prepareQuery(cf_UserNotificationsSettings)
-					.getKeySlice(userid)
-					.execute();
-		} catch (ConnectionException e) {
-			e.printStackTrace();
-		}
-		//if there are no settings create them and put all of them at true
-		if (result.getResult().getRowByIndex(0).getColumns().size() == 0) {
-			_log.info("Userid " + userid + " settings not found, initiating its settings...");
-			for (int i = 0; i < NotificationChannelType.values().length; i++) {
-				setUserNotificationChannel(userid, NotificationChannelType.values()[i], true);
-				toReturn.add(NotificationChannelType.values()[i]);
+		NotificationChannelType[] toProcess = getUserNotificationPreferences(userid).get(notificationType);
+		if (toProcess == null) {
+			_log.warn("Single Notification preference of  " + userid +  " Type: " + notificationType + " not existing ... creating default");
+			return createNewNotificationType(userid, notificationType);
+		} else
+			for (int i = 0; i < toProcess.length; i++) {
+				toReturn.add(toProcess[i]);
 			}
+		return toReturn;
+	}
+	/**
+	 * called when you add new notification types where the setting does not exist yet
+	 * please note: by default we set only portal notifications
+	 */
+	private List<NotificationChannelType> createNewNotificationType(String userid, NotificationType notificationType) {
+		List<NotificationChannelType> toReturn = new ArrayList<NotificationChannelType>();
+		MutationBatch m = conn.getKeyspace().prepareMutationBatch();
+		String valueToInsert = "";
+		NotificationChannelType[] wpTypes = { NotificationChannelType.PORTAL };
+		
+		for (int i = 0; i < wpTypes.length; i++) {
+			valueToInsert += wpTypes[i];
+			if (i < wpTypes.length-1)
+				valueToInsert += ",";
+			toReturn.add(wpTypes[i]); //add the new added notification type
+		}
+		m.withRow(cf_UserNotificationsPreferences, userid).putColumn(notificationType.toString(), valueToInsert, null);
+		boolean overAllresult = execute(m);
+		if (overAllresult) {
+			_log.trace("Set New Notification Setting for " + userid + " OK");
 			return toReturn;
 		}
-		else {
-			for (Row<String, String> row : result.getResult()) 
-				for (Column<String> column : row.getColumns()) 
-					if (column.getStringValue().compareTo("true") == 0) toReturn.add(getChannelType(column.getName()));
-		}
-		return toReturn;
+		return new ArrayList<NotificationChannelType>(); //no notification if sth fails
 	}
 	/**
 	 * {@inheritDoc}
 	 */
 	@Override
-	public boolean setUserNotificationChannel(String userid, NotificationChannelType channel, boolean enabled) {
+	public boolean setUserNotificationPreferences(String userid, Map<NotificationType, NotificationChannelType[]> enabledChannels) {
 		MutationBatch m = conn.getKeyspace().prepareMutationBatch();
-		//check if the value exists and if so update it
+
+		for (NotificationType nType : enabledChannels.keySet()) {
+			String valueToInsert = "";
+			int channelsNo = (enabledChannels.get(nType) != null) ? enabledChannels.get(nType).length : 0;
+			for (int i = 0; i < channelsNo; i++) {
+				valueToInsert += enabledChannels.get(nType)[i];
+				if (i < channelsNo-1)
+					valueToInsert += ",";
+			}
+			if (channelsNo == 0) { //in case no channels were selected
+				valueToInsert = "";
+				_log.trace("No Channels selected for " + nType + " by " + userid);
+			}
+			m.withRow(cf_UserNotificationsPreferences, userid).putColumn(nType.toString(), valueToInsert, null);
+		}
+
+		boolean overAllresult = execute(m);
+		if (overAllresult)
+			_log.trace("Set Notification Map for " + userid + " OK");
+		else
+			_log.trace("Set Notification Map for " + userid + " FAILED");
+		return overAllresult;		
+	}
+	/**
+	 * {@inheritDoc}
+	 * 
+	 * by default Workspace and Calendar Notifications are set to Portal
+	 */
+	@Override
+	public Map<NotificationType, NotificationChannelType[]> getUserNotificationPreferences(String userid) throws NotificationTypeNotFoundException, NotificationChannelTypeNotFoundException {
+		_log.trace("Asking for Notification preferences of  " + userid);
+		Map<NotificationType, NotificationChannelType[]> toReturn = new HashMap<NotificationType, NotificationChannelType[]>();
 		OperationResult<Rows<String, String>> result = null;
 		try {
-			result = conn.getKeyspace().prepareQuery(cf_UserNotificationsSettings)
+			result = conn.getKeyspace().prepareQuery(cf_UserNotificationsPreferences)
 					.getKeySlice(userid)
 					.execute();
 		} catch (ConnectionException e) {
 			e.printStackTrace();
 		}
+		//if there are no settings for this user create an entry and put all of them at true
+		if (result.getResult().getRowByIndex(0).getColumns().size() == 0) {
+			_log.info("Userid " + userid + " settings not found, initiating its preferences...");
+			HashMap<NotificationType, NotificationChannelType[]> toCreate = new HashMap<NotificationType, NotificationChannelType[]>();
+
+			for (int i = 0; i < NotificationType.values().length; i++) {
+				//TODO: Potential bug in NotificationType for workspace are refactored
+				//create a map with all notification enabled except for workspace notifications (They start with WP_) it was the only quick way
+//				if (NotificationType.values()[i].toString().startsWith("WP_") || NotificationType.values()[i].toString().startsWith("CALENDAR")) {
+//					NotificationChannelType[] wpTypes = { NotificationChannelType.PORTAL };
+//					toCreate.put(NotificationType.values()[i], wpTypes);
+//				}
+//				else
+					toCreate.put(NotificationType.values()[i], NotificationChannelType.values());
+			}
+			setUserNotificationPreferences(userid, toCreate); //commit the map
+
+			return toCreate;
+		}
+		else {
+			_log.trace("Notification preferences Found for  " + userid);
+			for (Row<String, String> row : result.getResult()) 
+				for (Column<String> column : row.getColumns()) {
+					String[] channels = column.getStringValue().split(",");
+					NotificationChannelType[] toAdd = new NotificationChannelType[channels.length];		
+					for (int i = 0; i < channels.length; i++) {
+						if (channels[i].compareTo("") != 0)
+							toAdd[i] = (getChannelType(channels[i]));
+					}
+					toReturn.put(getNotificationType(column.getName()), toAdd);
 
-		boolean settingExists = false;
-		//if the setting exists and is different from enabled value update it
-		for (Row<String, String> row : result.getResult()) {
-			for (Column<String> column : row.getColumns()) 
-				if (column.getName().compareTo(channel.toString()) == 0 && (column.getBooleanValue() != enabled) ) { 					
-					m.withRow(cf_UserNotificationsSettings, userid).putColumn(channel.toString(), ""+enabled, null);
-					settingExists = true;
-					break;
 				}
 		}
-		if (! settingExists)
-			m.withRow(cf_UserNotificationsSettings, userid).putColumn(channel.toString(), ""+enabled, null);
-
-
-
-		boolean overAllresult = execute(m);
-		if (overAllresult)
-			_log.trace("Set Notification Channel for " + userid + " to " + enabled + " OK");
-		else
-			_log.error("Set Notification Channel for " + userid + " to " + enabled + " KO");
-		return overAllresult;		
+		return toReturn;
 	}
 	/*
 	 * 
@@ -1393,9 +1451,30 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 		else if (type.compareTo("LIKE") == 0) {
 			return NotificationType.LIKE;
 		}
+		else if (type.compareTo("CALENDAR_ADDED_EVENT") == 0) {
+			return NotificationType.CALENDAR_ADDED_EVENT;
+		}
+		else if (type.compareTo("CALENDAR_UPDATED_EVENT") == 0) {
+			return NotificationType.CALENDAR_UPDATED_EVENT;
+		}
+		else if (type.compareTo("CALENDAR_DELETED_EVENT") == 0) {
+			return NotificationType.CALENDAR_DELETED_EVENT;
+		}
+		else if (type.compareTo("CALENDAR_ADDED_EVENT") == 0) {
+			return NotificationType.CALENDAR_ADDED_EVENT;
+		}
+		else if (type.compareTo("CALENDAR_UPDATED_EVENT") == 0) {
+			return NotificationType.CALENDAR_UPDATED_EVENT;
+		}
+		else if (type.compareTo("CALENDAR_DELETED_EVENT") == 0) {
+			return NotificationType.CALENDAR_DELETED_EVENT;
+		}
 		else if (type.compareTo("MESSAGE") == 0) {
 			return NotificationType.MESSAGE;
 		}
+		else if (type.compareTo("POST_ALERT") == 0) {
+			return NotificationType.POST_ALERT;
+		}
 		else if (type.compareTo("REQUEST_CONNECTION") == 0) {
 			return NotificationType.REQUEST_CONNECTION;
 		}
@@ -1425,7 +1504,7 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 		}
 		else if (type.compareTo("DOCUMENT_WORKFLOW_FIRST_STEP_REQUEST_INVOLVMENT") == 0) {
 			return NotificationType.DOCUMENT_WORKFLOW_FIRST_STEP_REQUEST_INVOLVMENT;
-		}
+		}		
 		else if (type.compareTo("GENERIC") == 0) {
 			return NotificationType.GENERIC;
 		}
@@ -1501,4 +1580,5 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 	}
 
 
+
 }
